@@ -2,7 +2,9 @@ import pytest
 
 import requests_mock
 from requests_mock import NoMockAddress
+from test_hooks import ExampleResponseDataclass
 
+from apitist import ResponseDataclassConverterHook
 from apitist import decorators as deco
 from apitist import session
 from apitist.requests import SharedSession
@@ -11,6 +13,7 @@ from apitist.requests import SharedSession
 class ExampleClient:
     def __init__(self, hostname):
         self.session = session(hostname)
+        self.session.add_hook(ResponseDataclassConverterHook)
 
     @deco.get("/get")
     def get(self):
@@ -24,6 +27,10 @@ class ExampleClient:
     def get_id_and_query(self, id, well=None):
         ...
 
+    @deco.get("/get?id={}", name="Get name with {id}")
+    def get_with_name(self, id):
+        ...
+
     @deco.delete("/delete")
     def delete(self):
         ...
@@ -50,6 +57,14 @@ class ExampleClient:
             "data": "hello world",
         }
 
+    @deco.post("/post", structure_type=ExampleResponseDataclass)
+    def post_with_converter(self):
+        ...
+
+    @deco.post("/post", structure_type=SharedSession)
+    def post_with_wrong_converter(self, **kwargs):
+        return dict(**kwargs)
+
     @deco.put("/put")
     def put(self, json=None):
         return {"json": json}
@@ -154,6 +169,29 @@ class TestRequests:
             res = client.get_id_and_query(1, well="known")
             assert res.text == "mocked"
 
+    def test_structuring(self):
+        host = "https://httpbin.org"
+        client = ExampleClient(host)
+
+        res = client.post_with_converter()
+        assert isinstance(res.data, ExampleResponseDataclass)
+
+    def test_structuring_override(self):
+        host = "https://httpbin.org"
+        client = ExampleClient(host)
+
+        res = client.post_with_wrong_converter(
+            structure_type=ExampleResponseDataclass
+        )
+        assert isinstance(res.data, ExampleResponseDataclass)
+
+    def test_request_name(self):
+        host = "https://httpbin.org"
+        client = ExampleClient(host)
+
+        res = client.get_with_name(1)
+        assert res.request.name == "Get name with {id}"
+
     def test_shared_state(self):
         s1 = session("https://google.com")
         s2 = session("https://yandex.ru")
@@ -168,3 +206,39 @@ class TestRequests:
         s1.get("/?q=124")
 
         assert s1.cookies == s2.cookies
+
+    def test_shared_state_incorrect_session(self):
+        s1 = session("https://google.com")
+        s2 = "123"
+
+        ss = SharedSession(s1)
+        with pytest.raises(ValueError):
+            ss.add_sessions(s2)
+
+    def test_non_class_function(self):
+        @deco.get("/test")
+        def test():
+            ...
+
+        with pytest.raises(Exception):
+            test()
+
+    def test_class_miss_session(self):
+        class Test:
+            @deco.get("/test")
+            def test(self):
+                ...
+
+        with pytest.raises(Exception):
+            Test().test()
+
+    def test_class_wrong_modificators_returned(self):
+        class Test:
+            session = session("https://httpbin.org")
+
+            @deco.get("/get")
+            def test(self):
+                return list()
+
+        with pytest.raises(Exception):
+            Test().test()
